@@ -14,7 +14,7 @@ class EmailDeviceAdmin(admin.ModelAdmin):
             'fields': ['user', 'name', 'confirmed'],
         }),
         ('Configuration', {
-            'fields': ['key'],
+            'fields': ['email'],
         }),
     ]
     raw_id_fields = ['user']
@@ -22,6 +22,6 @@ class EmailDeviceAdmin(admin.ModelAdmin):
 
 # Somehow this is getting imported twice, triggering a useless exception.
 try:
-    admin.site.register(EmailDevice)
+    admin.site.register(EmailDevice, EmailDeviceAdmin)
 except AlreadyRegistered:
     pass
