@@ -1,19 +1,27 @@
 from datetime import timedelta
 from doctest import DocTestSuite
 import pickle
+from threading import Thread
 import unittest
 
 from freezegun import freeze_time
 
 from django.contrib.auth import get_user_model
-from django.db import IntegrityError
+from django.contrib.auth.models import AnonymousUser
+from django.db import IntegrityError, connection
 from django.test import RequestFactory
 from django.test import TestCase as DjangoTestCase
+from django.test import TransactionTestCase as DjangoTransactionTestCase
+from django.test import skipUnlessDBFeature
+from django.test.utils import override_settings
 from django.urls import reverse
+from django.utils import timezone
 
-from django_otp import DEVICE_ID_SESSION_KEY, oath, util
+from django_otp import DEVICE_ID_SESSION_KEY, match_token, oath, user_has_device, util, verify_token
+from django_otp.forms import OTPTokenForm
 from django_otp.middleware import OTPMiddleware
 from django_otp.models import VerifyNotAllowed
+from django_otp.plugins.otp_static.models import StaticDevice
 
 
 def load_tests(loader, tests, pattern):
@@ -26,7 +34,14 @@ def load_tests(loader, tests, pattern):
     return suite
 
 
-class TestCase(DjangoTestCase):
+class TestThread(Thread):
+    "Django testing quirk: threads have to close their DB connections."
+    def run(self):
+        super().run()
+        connection.close()
+
+
+class OTPTestCaseMixin:
     """
     Utilities for dealing with custom user models.
     """
@@ -47,6 +62,14 @@ class TestCase(DjangoTestCase):
         return self.User.objects.create_user(username, password=password, **kwargs)
 
 
+class TestCase(OTPTestCaseMixin, DjangoTestCase):
+    pass
+
+
+class TransactionTestCase(OTPTestCaseMixin, DjangoTransactionTestCase):
+    pass
+
+
 class ThrottlingTestMixin:
     """
     Generic tests for throttled devices.
@@ -105,11 +128,14 @@ class ThrottlingTestMixin:
         self.assertEqual(data1, None)
 
         # After failure, verify is not allowed
-        self.device.verify_token(self.invalid_token())
-        verify_is_allowed2, data2 = self.device.verify_is_allowed()
-        self.assertEqual(verify_is_allowed2, False)
-        self.assertEqual(data2, {'reason': VerifyNotAllowed.N_FAILED_ATTEMPTS,
-                                 'failure_count': 1})
+        with freeze_time():
+            self.device.verify_token(self.invalid_token())
+            verify_is_allowed2, data2 = self.device.verify_is_allowed()
+            self.assertEqual(verify_is_allowed2, False)
+            self.assertEqual(data2, {'reason': VerifyNotAllowed.N_FAILED_ATTEMPTS,
+                                     'failure_count': 1,
+                                     'locked_until': timezone.now() + timezone.timedelta(seconds=1)
+                                     })
 
         # After a successful attempt, should be allowed again
         with freeze_time() as frozen_time:
@@ -121,6 +147,43 @@ class ThrottlingTestMixin:
             self.assertEqual(data3, None)
 
 
+@override_settings(OTP_STATIC_THROTTLE_FACTOR=0)
+class APITestCase(TestCase):
+    def setUp(self):
+        try:
+            self.alice = self.create_user('alice', 'password')
+            self.bob = self.create_user('bob', 'password')
+        except IntegrityError:
+            self.skipTest("Unable to create a test user.")
+        else:
+            device = self.alice.staticdevice_set.create()
+            device.token_set.create(token='alice')
+
+    def test_user_has_device(self):
+        with self.subTest(user='anonymous'):
+            self.assertFalse(user_has_device(AnonymousUser()))
+        with self.subTest(user='alice'):
+            self.assertTrue(user_has_device(self.alice))
+        with self.subTest(user='bob'):
+            self.assertFalse(user_has_device(self.bob))
+
+    def test_verify_token(self):
+        device = self.alice.staticdevice_set.first()
+
+        verified = verify_token(self.alice, device.persistent_id, 'bogus')
+        self.assertIsNone(verified)
+
+        verified = verify_token(self.alice, device.persistent_id, 'alice')
+        self.assertIsNotNone(verified)
+
+    def test_match_token(self):
+        verified = match_token(self.alice, 'bogus')
+        self.assertIsNone(verified)
+
+        verified = match_token(self.alice, 'alice')
+        self.assertEqual(verified, self.alice.staticdevice_set.first())
+
+
 class OTPMiddlewareTestCase(TestCase):
     def setUp(self):
         self.factory = RequestFactory()
@@ -292,3 +355,90 @@ class LoginViewTestCase(TestCase):
 
         response = self.client.get('/')
         self.assertContains(response, self.alice.get_username())
+
+
+@skipUnlessDBFeature('has_select_for_update')
+@override_settings(OTP_STATIC_THROTTLE_FACTOR=0)
+class ConcurrencyTestCase(TransactionTestCase):
+    def setUp(self):
+        try:
+            self.alice = self.create_user('alice', 'password')
+            self.bob = self.create_user('bob', 'password')
+        except IntegrityError:
+            self.skipTest("Unable to create a test user.")
+        else:
+            for user in [self.alice, self.bob]:
+                device = user.staticdevice_set.create()
+                device.token_set.create(token='valid')
+
+    def test_verify_token(self):
+        class VerifyThread(Thread):
+            def __init__(self, user, device_id, token):
+                super().__init__()
+
+                self.user = user
+                self.device_id = device_id
+                self.token = token
+
+                self.verified = None
+
+            def run(self):
+                self.verified = verify_token(self.user, self.device_id, self.token)
+                connection.close()
+
+        device = self.alice.staticdevice_set.get()
+        threads = [VerifyThread(device.user, device.persistent_id, 'valid') for _ in range(10)]
+        for thread in threads:
+            thread.start()
+        for thread in threads:
+            thread.join()
+
+        self.assertEqual(sum(1 for t in threads if t.verified is not None), 1)
+
+    def test_match_token(self):
+        class VerifyThread(Thread):
+            def __init__(self, user, token):
+                super().__init__()
+
+                self.user = user
+                self.token = token
+
+                self.verified = None
+
+            def run(self):
+                self.verified = match_token(self.user, self.token)
+                connection.close()
+
+        threads = [VerifyThread(self.alice, 'valid') for _ in range(10)]
+        for thread in threads:
+            thread.start()
+        for thread in threads:
+            thread.join()
+
+        self.assertEqual(sum(1 for t in threads if t.verified is not None), 1)
+
+    def test_concurrent_throttle_count(self):
+        self._test_throttling_concurrency(thread_count=10, expected_failures=10)
+
+    @override_settings(OTP_STATIC_THROTTLE_FACTOR=1)
+    def test_serialized_throttling(self):
+        # After the first failure, verification will be skipped and the count
+        # will not be incremented.
+        self._test_throttling_concurrency(thread_count=10, expected_failures=1)
+
+    def _test_throttling_concurrency(self, thread_count, expected_failures):
+        forms = (
+            OTPTokenForm(device.user, None, {'otp_device': device.persistent_id, 'otp_token': 'bogus'})
+            for _ in range(thread_count)
+            for device in StaticDevice.objects.all()
+        )
+
+        threads = [TestThread(target=form.is_valid) for form in forms]
+        for thread in threads:
+            thread.start()
+        for thread in threads:
+            thread.join()
+
+        for device in StaticDevice.objects.all():
+            with self.subTest(user=device.user.get_username()):
+                self.assertEqual(device.throttling_failure_count, expected_failures)
